#
# eda.py
#
# receive and display eda
# eda.py serial_port
#
# Eyal Perry
# CBA MIT 11/4/19
#
# (c) Massachusetts Institute of Technology 2011
# Permission granted for experimental and personal use;
# license for commercial sale available from MIT
#

from Tkinter import *
import serial
import numpy as np

WINDOW = 600 # window size
load_eps = 0.5 # filter fraction
eda_eps = 0.75
DIFF_BUFFER_SIZE = 50

last_loads = np.array([0.0 for i in range(DIFF_BUFFER_SIZE)])
last_edas = np.array([0.0 for i in range(DIFF_BUFFER_SIZE)])
A = np.zeros((100, 2), dtype=float)
b = np.zeros(100, dtype=float)
resid = 0.
params = [0.0, 0.0]
A[:, 1] = 1.
load_filter = 0.0
eda_filter = 0.0 # filtered value
load_to_eda_factor = 0.85

def idle(parent,canvas):
   global resid, params, last_loads, last_eda, load_filter, eda_filter, load_eps, eda_eps, load_to_eda_factor, A, b
   #
   # idle routine
   #
   byte2 = 0
   byte3 = 0
   byte4 = 0
   ser.flush()
   #
   # find framing
   #
   while 1:
      byte1 = byte2
      byte2 = byte3
      byte3 = byte4
      byte4 = ord(ser.read())
      if ((byte1 == 1) & (byte2 == 2) & (byte3 == 3) & (byte4 == 4)):
         break
   #
   # read and plot
   #
   load_up_low = ord(ser.read())
   load_up_high = ord(ser.read())
   load_down_low = ord(ser.read())
   load_down_high = ord(ser.read())
   eda_low = ord(ser.read())
   eda_high = ord(ser.read())

   load_up_value = 256*load_up_high + load_up_low
   load_down_value = 256*load_down_high + load_down_low
   load_value = (load_up_value + (1023 - load_down_value))/2.0
   A[:-1, 0] = A[1:, 0]
   A[-1, 0] = load_value
   load_filter = (1-load_eps)*load_filter + load_eps*load_value
   load_diff = load_filter - np.mean(last_loads)
   last_loads[:-1] = last_loads[1:]
   last_loads[-1] = load_filter
   load_x = int(.55*WINDOW + (.9-.2)*WINDOW*load_diff/512.0)
   canvas.itemconfigure("text1",text="dLoad: %.1f"%load_diff)
   canvas.coords('rect11',.2*WINDOW,.05*WINDOW,load_x,.2*WINDOW)
   canvas.coords('rect12',load_x,.05*WINDOW,.9*WINDOW,.2*WINDOW)

   eda_value = 256*eda_high + eda_low
   b[:-1] = b[1:]
   b[-1] = eda_value
   eda_filter = (1-eda_eps)*eda_filter + eda_eps*eda_value
   eda_diff = eda_filter - np.mean(last_edas)
   last_edas[:-1] = last_edas[1:]
   last_edas[-1] = eda_filter
   #eda_diff = np.max(last_edas) - np.min(last_edas)
   eda_x = int(.55*WINDOW + (.9-.2)*WINDOW*eda_diff/512.0)
   canvas.itemconfigure("text2",text="dEDA: %.1f"%eda_diff)
   canvas.coords('rect21',.2*WINDOW,.3*WINDOW,eda_x,.45*WINDOW)
   canvas.coords('rect22',eda_x,.3*WINDOW,.9*WINDOW,.45*WINDOW)

   ols = np.linalg.lstsq(A, b)

   if ols[3][1] > 1e-1:
      params = ols[0]
      resid = ols[1]
      print("params", ols[0])
      print("resid", ols[1])
      print("sing", ols[3])

   if resid > 0:
       a = (load_value * params[0] + params[1] - eda_value) ** 2
       print(a)
       norm_eda = a * 100 / resid
   else:
       norm_eda = 0
   fear = norm_eda
   #norm_eda = max(0, eda_diff + min(0, load_diff))
   #fear = 4 * norm_eda *norm_eda
   fear_x = int(.2*WINDOW + (.9-.2)*WINDOW*fear/512.0)
   canvas.itemconfigure("text3",text="Fear: %.1f"%fear)
   canvas.coords('rect31',.2*WINDOW,.55*WINDOW,fear_x,.7*WINDOW)
   canvas.coords('rect32',fear_x,.55*WINDOW,.9*WINDOW,.7*WINDOW)

   canvas.update()
   parent.after_idle(idle,parent,canvas)

#
#  check command line arguments
#
if (len(sys.argv) != 2):
   print "command line: eda_norm.py serial_port"
   sys.exit()
port = sys.argv[1]
#
# open serial ports
#
ser = serial.Serial(port,115200)
ser.setDTR()
#
# set up GUI
#
root = Tk()
root.title('eda.py (q to exit)')
root.bind('q','exit')
canvas = Canvas(root, width=WINDOW, height=.75*WINDOW, background='white')
#
canvas.create_text(.1*WINDOW,.125*WINDOW,text="Load",font=("Helvetica", 14),tags="text1",fill="#0000b0")
canvas.create_rectangle(.2*WINDOW,.05*WINDOW,.3*WINDOW,.2*WINDOW, tags='rect11', fill='#b00000')
canvas.create_rectangle(.3*WINDOW,.05*WINDOW,.9*WINDOW,.2*WINDOW, tags='rect12', fill='#0000b0')
#
canvas.create_text(.1*WINDOW,.375*WINDOW,text="EDA",font=("Helvetica", 14),tags="text2",fill="#0000b0")
canvas.create_rectangle(.2*WINDOW,.3*WINDOW,.3*WINDOW,.45*WINDOW, tags='rect21', fill='#b00000')
canvas.create_rectangle(.3*WINDOW,.3*WINDOW,.9*WINDOW,.45*WINDOW, tags='rect22', fill='#0000b0')
#
canvas.create_text(.1*WINDOW,.625*WINDOW,text="Norm EDA",font=("Helvetica", 14),tags="text3",fill="#0000b0")
canvas.create_rectangle(.2*WINDOW,.55*WINDOW,.3*WINDOW,.7*WINDOW, tags='rect31', fill='#b00000')
canvas.create_rectangle(.3*WINDOW,.55*WINDOW,.9*WINDOW,.7*WINDOW, tags='rect32', fill='#0000b0')
canvas.pack()
#
# start idle loop
#
root.after(100,idle,root,canvas)
root.mainloop()
